/* -*-java-*-
 * T. Litt June 2010
 * Drag and drop for wiki attach.
 */

$(function() {
    /* Check for Google Gears. */
    if (!window.google || !google.gears || typeof(google.gears.factory.create) == 'undefined') {
	/* display error zone, remove drop zone */
      $('#fail').show();
      $('#dropzone').css('display','none');
      return;
    }
    
    /* Google Gears support. */
    var desktop = google.gears.factory.create('beta.desktop');
    var FILES = 'application/x-gears-files';
    var idx;
    var data = new( Object );
    var request;

    /* Colors (could be css clases) */

    var DropZoneReadyColor = '#ead1b2';
    var DropZoneActiveColor = '#90ee90';

    /* default is to refuse drops.  Thus, preventDefault enables them. */

    function preventDefault(evt) {
	if (evt.preventDefault)
	    evt.preventDefault();
	evt.returnValue = false;
    }

    /* Determine if all files in data are permitted to be dropped. */

    function validFiles(data) {
	var accept = (data && data.count);

	/* This is harder than it looks, as browsers vary in how much effort they
         * put into determing mime types.  This mix of tests enabled an implementation
	 * to allow images (except icons).  The actual regexps are external to this file,
	 * and can be adjusted as desired.  An easy way to accept anything is to 
	 * set FileRejectExtensions to /^\b$/ and FileAcceptExtensions to /^\.\w+/
	 */

	if( accept ) {
	    /* Reject if we know any file is bad */
	    for( var i in data.extensions ) {
		/* Check for extension over-rides first */

		/* Reject based on extension */
		if( data.extensions[i] && data.extensions[i].match( FileRejectExtensions ) ) {
		    accept = 0;
		    break;
		}

		/* Force these to accept even if mimeType denys (is wrong) */
		if( data.extensions[i] && data.extensions[i].match( FileAcceptExtensions ) ) {
		    continue;
		}

		/* Reject if forced or unacceptable */
		if( data.mimeTypes[i] && (data.mimeTypes[i].match( FileRejectMime ) || !data.mimeTypes[i].match(FileAcceptMime)) ) {
		    accept = 0;
		    break;
		}
	    }
	}

	return accept;
    }

    var dragEnter = dragOver =
	function(event) { 
	    var evt = event || window.event; 
	    var data = desktop.getDragData(evt, FILES);

	    var accept = validFiles(data);
	    desktop.setDropEffect(evt, (accept? 'copy' : 'none') );

	    $('#dropzone').css("background-color", (accept? DropZoneActiveColor : DropZoneReadyColor) ); 

	    if (!navigator.userAgent.match( /Firefox\/[2,3]\.0/) ) {
		preventDefault(evt);
	    } else if (!accept) {
		preventDefault(evt);
	    }
	};

    var dragLeave =
	function(event) { 
	    var evt = event || window.event; 

	    var data = desktop.getDragData(event, FILES);

	    desktop.setDropEffect(evt, (validFiles(data)? 'copy' : 'none') );

	    $('#dropzone').css("background-color", DropZoneReadyColor); 
	};

    function addEvent(element, type, handler) {
	if (element.addEventListener) {
	    element.addEventListener(type, handler, false);
	} else if (element.attachEvent) {
	    element.attachEvent('on' + type, handler);
	} else {
	    alert( "Unable to add drag and drop event " + type );
	}
    }

    var html5 = true;
    if (navigator.userAgent.match( /Firefox\/[2,3]\.0/ )) {
	html5 = false;
    }

    var elem = $('#dropzone').get(0);

    addEvent(elem, 'dragenter', dragEnter);
    addEvent(elem, 'dragover', dragOver);
    addEvent(elem, html5 ? 'dragleave' : 'dragexit', dragLeave);
    addEvent(elem, html5 ? 'drop' : 'dragdrop', upload);

    /* Add a form item to a POST request */

    function postFormItem( builder, boundary, dashdash, crlf, name, value ) {

	/* Item header */
        builder.append(crlf); 
        builder.append(dashdash);
        builder.append(boundary);
        builder.append(crlf); 

        builder.append('Content-Disposition: form-data; name="' + name + '"');
        builder.append(crlf); 
        builder.append(crlf); 

	/* Item value */
        builder.append(value); 

	return;
    }

    /* Add a file to a POST request */

    function postFile( builder, boundary, dashdash, crlf, name, file, type ) {

	/* Header */
	builder.append(crlf);
	builder.append(dashdash);
	builder.append(boundary);
	builder.append(crlf);

        builder.append('Content-Disposition: form-data; name="' + name + '"');
	if (file.name) {
	    builder.append('; filename="' + file.name + '"');
	}
	builder.append(crlf);
            
        builder.append('Content-Type: ' + type);
        builder.append(crlf);
        builder.append(crlf); 
            
        /* File data (binary) */
        builder.append(file.blob);
    
	return;
   }

    /* Make (and post) an http(s) request for the next file */

    function makereq() {

        var boundary = '------manypartformboundary' + (new Date).getTime() + Math.floor(Math.random()*1073741826).toString();
        var dashdash = '--';
        var crlf     = '\r\n';

	/* Show spinner for each dropped file. */
        $('#dropfile').html( '<img src="' + PubUrlPath + '/TWiki/TWikiAttach/spinner.gif" width="16" height="16" />' 
			    + data.files[idx].name + '  (' + (idx+1) + ' of ' + data.count + ' files)');

	/* Build RFC2388 string. */
	var builder = google.gears.factory.create('beta.blobbuilder');

	/* Send file */
	postFile(builder, boundary, dashdash, crlf, 'filepath', data.files[idx], 'application/octet-stream' );
            
	/* Form data */
	/* Mark as dropped files */
	postFormItem( builder, boundary, dashdash, crlf, 'DroppedFiles', 'true' );

	/* Send file type - as best we know it */

	postFormItem( builder, boundary, dashdash, crlf, 'FileType', 
		      (data.mimeTypes[idx]? data.mimeTypes[idx] : 'application/octet-stream') );

	/* Add items from standard form */
	var f = document.getElementById("mainattachform");
	if( f.hidefile.checked ) {
	    postFormItem( builder, boundary, dashdash, crlf, 'hidefile', 'true' );
	}	
	postFormItem( builder, boundary, dashdash, crlf, 'filename', f.filename.value );
	if( f.filecomment.value.length != 0 ) {
	    postFormItem( builder, boundary, dashdash, crlf, 'filecomment', f.filecomment.value );
	}
	if( f.createlink.checked ) {
	    postFormItem( builder, boundary, dashdash, crlf, 'createlink', 'true' );
	}
	/* End of post boundary */
        builder.append(crlf);
        builder.append(dashdash);
        builder.append(boundary);
	builder.append(dashdash);  /* Mark as end of POST data */
        builder.append(crlf); 

	/* HTTP request object */

	request = google.gears.factory.create('beta.httprequest');

	request.upload.onprogress = function(e) {
	    if( e.lengthComputable ) {
		var donepct = 0;
		if( e.total != 0 ) {
		    donepct  = Math.round((e.loaded/e.total)*100);
		}
		if( donepct < 99 ) {
		    $('#dropratio').html( e.loaded + ' bytes sent / ' + e.total + ' = ' + donepct.toString() + '%' );
		} else {
		    $('#droplbl').html( '<b>Status: </b>' );
		    $('#dropratio').html( 'Server is processing ' + data.files[idx].name + ', please be patient.'  );
		}
	    } else {
		$('#dropratio').html( e.loaded + ' bytes sent / total to be sent is not known' );
	    }
	};
        
	request.onreadystatechange = function() {

	    switch(request.readyState) {
                case 4:
		    /* Stop the spinner */

		    $('#dropfile').html( '<span style="padding-left:16px;"' + data.files[idx].name + '  (' + (idx+1) + ' of ' + data.count + ' files)' );

                    /* If we got a response display it. */
		    if( request.status == 200 ) {
			if (request.responseText) {
			    var rsp = request.responseText.replace( /\n/mg,'<newline>' );
			    rsp = rsp.replace( /^.*<body[^>]*>/i,'' );
			    rsp = rsp.replace( /<\/body.*$/i, '' );
			    rsp = rsp.replace( /<newline>/mg, "\n" );
			    $('#dropresponse').append( rsp );
			} else {
			    $('#dropresponse').append( 'No response but status was OK for ' + data.files[idx].name );
			}
		    } else {
			$('#dropresponse').append( 'Error communicating with server: ' + request.status.toString() + ' ' + request.statusText );
		    }

                    if( ++idx >= data.count ) {
			/* Done */
			$('#droplbl').html( '<b>Status: </b>' );
			$('#dropratio').html( 'All files have been uploaded.'  );
		    } else {
			/* POST next file */
			makereq();
		    }
                   break;
                }
	    };

	/* Display progress area */        
	$('#dropprog').css("display", 'block');

	/* Submit request.*/
	request.open("POST", UploadPath );
	request.setRequestHeader('content-type', 'multipart/form-data; boundary=' + boundary);
	request.send(builder.getAsBlob());
    } /* makereq */

    /* Handle drop event */

    function upload(event) {

	var evt = event || window.event;
        data = desktop.getDragData(event, FILES);

	if( ! (data != null && data.files) ) {
	    alert( "You have encountered a bug in google gears, which is used by this application.\n\nSorry for the inconvenience.\n\nTry completely closing your browser and opening a new copy.\n\nAlternatively, you can see if a newer version of google gears is available at http://gears.google.com?action=install" );
	    return;
	}
	if( !validFiles(data) ) {
	    /* If we did the right thing on drag enter/over, this shouldnt happen! */
	    desktop.setDropEffect(evt, 'none');
	    alert( "Drop contains no files or invalid files" );
	    return;
	}
	desktop.setDropEffect(evt, 'copy');

        /* Clear status from any previous transfer */

	$('#droplbl').html( '<b>Upload progress: </b>' );
	$('#dropratio').html( '&nbsp;'  );
      
 	$('#dropresponse').html( '&nbsp;' );
	$('#dropfile').html( '&nbsp');

        /* Handle each file as a separate post request.
	 * This is done to limit impact on the server as well as to provide
         * the most timely feedback to the user.  Note that SSL buffering tends
         * to delay feedback.
	 */
	idx = 0;
	makereq();

        /* Prevent FireFox opening the dragged file. */
	if ($.browser.mozilla) {
	    event.stopPropagation();
	}
        
        /* Fix Safari forgetting the background change after drop. */
        $('#dropzone').css("background-color", DropZoneReadyColor);
    }
});
